/*
  Copyright (c) 2008-2009 LordZen

  This file is part of tinycalc.

  tinycalc is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tinycalc is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with tinycalc.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "lex.h"
#include "debug.h"

Lex *lex_init(char *exp)
{
  Lex *lex = (Lex *)malloc(sizeof(Lex));

  if(lex != NULL)
  {
    lex->exp = (char *)exp;
    lex->pos = 0;
    lex->token = 0;

		show_debug("syntax analizator initilized");
    
		return lex;
  }
  else
    return NULL;
}

Token lex_next_token(Lex *lex)
{
  //skip white space
  while(lex->exp[lex->pos] == ' ' || lex->exp[lex->pos] == '\t')
    lex->pos++;

  if(lex->exp[lex->pos] == '\0')
  {
    lex->token = EOS;
    strcpy(lex->value,"");
    return EOS;
  }
  else if(lex->exp[lex->pos] == '+')
  {
    lex->token = SUM;
    strcpy(lex->value,"+");
    lex->pos++;
    return SUM;
  }
  else if(lex->exp[lex->pos] == '-')
  {
    lex->token = SUB;
    strcpy(lex->value,"-");
    lex->pos++;
    return SUB;
  }
  else if(lex->exp[lex->pos] == '*')
  {
    lex->token = MUL;
    strcpy(lex->value,"*");
    lex->pos++;
    return MUL;
  }
  else if(lex->exp[lex->pos] == '/')
  {
    lex->token = DIV;
    strcpy(lex->value,"/");
    lex->pos++;
    return DIV;
  }
  else if(lex->exp[lex->pos] == '(')
  {
    lex->token = OPAR;
    strcpy(lex->value,"(");
    lex->pos++;
    return OPAR;
  }
  else if(lex->exp[lex->pos] == ')')
  {
    lex->token = CPAR;
    strcpy(lex->value,")");
    lex->pos++;
    return CPAR;
  }
  else if(isdigit(lex->exp[lex->pos]))  //number
  {
    //this buffer contains the current token value
    char token_value[LEX_BSIZE+1];

    //token buffer pointer
    int i = 0;
    
    if(lex->exp[lex->pos] == '0')
    {
      token_value[i++] = lex->exp[lex->pos++];
      
      if(tolower(lex->exp[lex->pos]) == 'x') //esadecimal number
      {
	token_value[i++] = lex->exp[lex->pos++];
	
	while(i < LEX_BSIZE && (isdigit(lex->exp[lex->pos]) || 
				(tolower(lex->exp[lex->pos]) >= 'a' && tolower(lex->exp[lex->pos]) <= 'f')))
	  token_value[i++] = lex->exp[lex->pos++];
	
	token_value[i] = '\0';
	lex->token = HEX;
	strcpy(lex->value,token_value);
	return HEX;
      }
      else if(tolower(lex->exp[lex->pos]) == 'b') //binary number
      {
	token_value[i++] = lex->exp[lex->pos++];

	while(i < LEX_BSIZE && (lex->exp[lex->pos] == '0' || lex->exp[lex->pos] == '1'))
	  token_value[i++] = lex->exp[lex->pos++];
	
	token_value[i] = '\0';
	lex->token = BIN;
	strcpy(lex->value,token_value);
	return BIN;
      }
      else if(isdigit(lex->exp[lex->pos])) //decimal number
      {
	token_value[i++] = lex->exp[lex->pos++];

	while(i < LEX_BSIZE && isdigit(lex->exp[lex->pos]))
	  token_value[i++] = lex->exp[lex->pos++];
	
	token_value[i] = '\0';
	lex->token = DEC;
	strcpy(lex->value,token_value);
	return DEC;
      }
      else
	return ERR; //unknow token
    }
    else if(isdigit(lex->exp[lex->pos])) //decimal number
    {
      token_value[i++] = lex->exp[lex->pos++];

      while(i < LEX_BSIZE && isdigit(lex->exp[lex->pos]))
	  token_value[i++] = lex->exp[lex->pos++];

      token_value[i] = '\0';
      lex->token = DEC;
      strcpy(lex->value,token_value);
      return DEC;
    }
  }
  return ERR; //unknow token
}

void lex_free(Lex *lex)
{
  free(lex);
	show_debug("syntax analizator destroied");
}
