/*
  Copyright (c) 2008-2009 LordZen

  This file is part of tinycalc.

  tinycalc is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tinycalc is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with tinycalc.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>

#include "debug.h"
#include "convert.h"
#include "lex.h"
#include "parser.h"

long eval_exp(Lex *lex);
long eval_term(Lex *lex);
long eval_factor(Lex *lex);

long parse(Lex *lex)
{
  int result = eval_exp(lex);
  
  if(lex->token != EOS)
  {
    lex_free(lex);
    show_error("invalid expression");
  }
  return result;
}
 
long eval_exp(Lex *lex)
{
  long result,tmp;

  result = eval_term(lex);
  
  while(lex->token == SUM || lex->token == SUB)
  {
    show_debug(lex->value);

    if(lex->token == SUM)
    {
      tmp = eval_term(lex);
      result += tmp;
    }
    else if(lex->token == SUB)
    {
      tmp = eval_term(lex);
      result -= tmp;
    }
  }
  
  return result;
}

long eval_term(Lex *lex)
{
  long result,tmp;
  
  result = eval_factor(lex);
  
  while(lex->token == MUL || lex->token == DIV)
  {
    show_debug(lex->value);

    if(lex->token == MUL)
    {
      tmp = eval_term(lex);
      result *= tmp;
    }
    else if(lex->token == DIV)
    {
      tmp = eval_term(lex);
      
      if(tmp == 0)
      {
	//free memory
	lex_free(lex);
	show_error("division by 0!!!");
      }
      result /= tmp;
    }
  }
  
  return result;
}

long eval_factor(Lex *lex)
{
  long result = 0;

  lex_next_token(lex);

  show_debug(lex->value);
  
  if(lex->token == OPAR)
  {
    result = eval_exp(lex);
    show_debug(lex->value);
    
    if(lex->token != CPAR)
    {
      //free memory
      lex_free(lex);
      show_error("parenthesis not closed");
    }
    lex_next_token(lex);
  }
  else if(lex->token == DEC)
  {
    result = atoi(lex->value);
    lex_next_token(lex); 
  }
  else if(lex->token == HEX)
  {
    result = hex2dec(lex->value);
    lex_next_token(lex); 
  }
  else if(lex->token == BIN)
  {
    result = bin2dec(lex->value);
    lex_next_token(lex); 
  } 
  else
  {
    lex_free(lex);
    show_error("invalid expression");
  }
    
  return result;
}
