/*
  Copyright (c) 2008-2009 LordZen

  This file is part of tinycalc.

  tinycalc is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tinycalc is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with tinycalc.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "config.h"
#include "debug.h"
#include "lex.h"
#include "convert.h"
#include "parser.h"

void usage(char *progname)
{
  printf("\n:: Tiny Calculator %s ::\n",PACKAGE_VERSION);
  printf("Copyright (c) 2008-2009 LordZen\n\n");
  printf("USAGE: %s <expression>\n",progname);
  printf("SAMPLE EXPRESSION: 0x2f+57-(0b1100001/4)\n\n");
  exit(0);
}

void show_debug(char *s)
{
#ifdef DEBUG
  printf("[*] DEBUG: %s\n",s);
#endif
}

void show_error(char *s)
{
  printf("\n[**] FATAL: %s\n\n",s);
  exit(-1);
}

long hex2dec(char *num)
{
  //skip 0x
  int i = 2;
  int pos = strlen(num) - 3;

  long curr_num = 0;
  long tot = 0;

  while(num[i] != '\0')
  {
    switch(num[i])
    {
      case '0':
            curr_num = 0;
            break;
      case '1':
            curr_num = 1;
            break;
      case '2':
            curr_num = 2;
            break;
      case '3':
            curr_num = 3;
            break;
      case '4':
            curr_num = 4;
            break;
      case '5':
            curr_num = 5;
            break;
      case '6':
            curr_num = 6;
            break;
      case '7':
            curr_num = 7;
            break;
      case '8':
            curr_num = 8;
            break;
      case '9':
            curr_num = 9;
            break;
      case 'a':
      case 'A':
            curr_num = 10;
            break;
      case 'b':
      case 'B':
            curr_num = 11;
            break;
      case 'c':
      case 'C':
            curr_num = 12;
            break;
      case 'd':
      case 'D':
            curr_num = 13;
            break;
      case 'e':
      case 'E':
            curr_num = 14;
            break;
      case 'f':
      case 'F':
            curr_num = 15;
            break;
    }
    tot += curr_num * pow(16,pos--);
    i++;
  }

  return tot;
}

long bin2dec(char *num)
{
  //skip 0b
  int i = 2;
  int pos = strlen(num) - 3;

  long curr_num = 0;
  long tot = 0;

  while(num[i] != '\0')
  {
    switch(num[i])
    {
      case '0':
            curr_num = 0;
            break;
      case '1':
            curr_num = 1;
            break;
    }
    tot += curr_num * pow(2,pos--);
    i++;
  }

  return tot;
}

void dec2bin(unsigned long num,char *bin)
{
  int i = 0;
  char buffer[MAX_BIT_BIN+1];

  //pointer to the end of buffer
  char *pbuf = buffer + sizeof(buffer) - 1;
  
  *pbuf-- = '\0';
  
  for(i=0; i<sizeof(num)*8; ++i)
  {
    unsigned long bit = (unsigned int)1 << i;
    *pbuf-- = ((num & bit)? '1' : '0');
  }

  *pbuf = ' ';
  
  //skip leading spaces and zeros
  while(*pbuf == '0' || *pbuf == ' ')
    pbuf++;
  
  strcpy(bin,pbuf);
}

void dec2hex(unsigned long num,char *hex)
{
  char buffer[MAX_BIT_HEX+1];
  
  //pointer to the end of buffer
  char *pbuf = buffer +sizeof(buffer) - 1;
  
  *pbuf-- = '\0';
  
  while(num != 0)
  {
    switch(num % 16)
    {
      case 0:
	*pbuf-- = '0';
	break;
      case 1:
	*pbuf-- = '1';
	break;
      case 2:
	*pbuf-- = '2';
	break;
      case 3:
	*pbuf-- = '3';
	break;
      case 4:
	*pbuf-- = '4';
	break;
      case 5:
	*pbuf-- = '5';
	break;
      case 6:
	*pbuf-- = '6';
	break;
      case 7:
	*pbuf-- = '7';
	break;
      case 8:
	*pbuf-- = '8';
	break;
      case 9:
	*pbuf-- = '9';
	break;
      case 10:
	*pbuf-- = 'a';
	break;
      case 11:
	*pbuf-- = 'b';
	break;
      case 12:
	*pbuf-- = 'c';
	break;
      case 13:
	*pbuf-- = 'd';
	break;
      case 14:
	*pbuf-- = 'e';
	break;
      case 15:
	*pbuf-- = 'f';
	break;
    }
    num /= 16;
  }

  pbuf++;
  
  strcpy(hex,pbuf);
}

int main(int argc,char *argv[])
{
  int decres = 0;
  char binres[MAX_BIT_BIN+1];
  char hexres[MAX_BIT_HEX+1];

  if(argc != 2)
    usage(argv[0]);

  Lex *lex = lex_init(argv[1]);

  if(lex == NULL)
    show_error("syntax analizator not initialized");

  decres = parse(lex);

  lex_free(lex);

  //convert result in binary format
  dec2bin((unsigned int)abs(decres),binres);

  //convert result in esadecimal format
  dec2hex((unsigned int)abs(decres),hexres);
  
  //print the result
  if(decres < 0)
    printf("\n\t%d\t\t\t-0x%s\t\t\t-0b%s\n\n",decres,hexres,binres);
  else
    printf("\n\t%d\t\t\t0x%s\t\t\t0b%s\n\n",decres,hexres,binres);

  return 0;
}
